indexing

	description:

		"Objects that create interpreters"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUT_INTERPRETER_GENERATOR

inherit

	ERL_G_GENERATOR
		redefine
			make,
			generate_all
		end

	AUT_SHARED_PATHNAMES
		export {NONE} all end

	AUT_SHARED_FILE_SYSTEM_ROUTINES
		export {NONE} all end

	KL_SHARED_EXECUTION_ENVIRONMENT
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make (a_universe: ET_UNIVERSE) is
			-- Create new reflection universe based on the gelint
			-- universe `a_universe'.
		local
			ise_eiffel_compiler: AUT_ISE_EIFFEL_COMPILER
			old_ise_eiffel_value: STRING
		do
			create ise_eiffel_compiler.make
			old_ise_eiffel_value := ise_eiffel_compiler.ise_eiffel_variable
			if
				old_ise_eiffel_value = Void and
				ise_eiffel_compiler.is_ise_eiffel_installed
			then
				execution_environment.set_variable_value (ise_eiffel_compiler.ise_eiffel_variable_name, ise_eiffel_compiler.ise_eiffel_path)
			end
			Precursor (a_universe)
			if old_ise_eiffel_value /= Void then
				execution_environment.set_variable_value (ise_eiffel_compiler.ise_eiffel_variable_name, old_ise_eiffel_value)
			end
		end

feature -- Access

	output_handler: PROCEDURE [ANY, TUPLE [STRING]]
			-- Output handler that will be invoked for any compiler output

	base_dirname: STRING
			-- Directory serving as base directory for relative directories in `universe'.

	last_interpreter: AUT_INTERPRETER_PROXY
			-- Proxy for the last interpreter created

	is_debug_mode_enabled: BOOLEAN
			-- Should the interpreter runtime be compiled with
			-- assertion checking on?

	external_text: STRING
		-- External clause of Ace file
		-- TODO: The Gobo Eiffel Lace parser does not yet provide
		-- information about the external clause. Until such support
		-- is provided the below workaround must stay.

feature -- Status setting

	set_output_handler (an_output_handler: like output_handler) is
			-- Set `output_handler' to `an_output_handler'.
		require
			an_output_handler_not_void: an_output_handler /= Void
		do
			output_handler := an_output_handler
		ensure
			output_handler_set: output_handler = an_output_handler
		end

	set_base_dirname (a_base_dirname: like base_dirname) is
				-- Set `base_dirname' to `a_base_dirname'.
		require
			a_base_dirname_not_void: a_base_dirname /= Void
		do
			base_dirname := a_base_dirname
		ensure
			base_dirname_set: base_dirname = a_base_dirname
		end

	set_external_text (an_external_text: like external_text) is
			-- Set `external_text' to `an_external_text'.
		do
			external_text := an_external_text
		ensure
			external_text_set: external_text = an_external_text
		end

	enable_debug_mode is
			-- Set `is_debug_mode_enabled' to `True'.
		require
			debug_mode_disabled: not is_debug_mode_enabled
		do
			is_debug_mode_enabled := True
		ensure
			is_debug_mode_enabled_set: is_debug_mode_enabled
		end

feature -- Generation

	generate_all (a_pathname: STRING) is
			-- Generate and compile interpreter in location `a_pathname'.
		local
			ise_eiffel_compiler: AUT_ISE_EIFFEL_COMPILER
			reflection_library_pathname: STRING
			absolute_pathname: STRING
		do
			if base_dirname = Void then
				base_dirname := a_pathname
			end
			absolute_pathname := file_system.absolute_pathname (a_pathname)
			reflection_library_pathname := file_system.pathname (a_pathname, "reflection_library")
			file_system.recursive_delete_directory (reflection_library_pathname)
			Precursor (reflection_library_pathname)
			file_system_routines.copy_recursive (pathnames.runtime_dirname, file_system.pathname (a_pathname, "runtime"))
			generate_ace_file (a_pathname)
			create ise_eiffel_compiler.make
			if output_handler /= Void then
				ise_eiffel_compiler.set_output_handler (output_handler)
			end
			ise_eiffel_compiler.run (a_pathname, ace_filename, interpreter_system_name)
		end

	create_interpreter (a_pathname: STRING; a_log_dirname: STRING) is
			-- Create interpreter proxy based on executable found in `a_pathname'
			-- and make it available via `last_interpreter'.
		require
			a_pathname_not_void: a_pathname /= Void
			a_pathname_not_empty: not a_pathname.is_empty
			a_log_dirname_not_void: a_log_dirname /= Void
			a_log_dirname_not_empty: not a_log_dirname.is_empty
		local
			absolute_pathname: STRING
			executable_filename: STRING
		do
			absolute_pathname := file_system.absolute_pathname (a_pathname)
 			file_system.recursive_create_directory (a_log_dirname)
			executable_filename := file_system.nested_pathname (absolute_pathname, <<"EIFGEN", "F_code", interpreter_filename>>)
			if file_system.file_exists (executable_filename) then
				create last_interpreter.make (executable_filename,
												universe,
												file_system.pathname (a_log_dirname, "interpreter_log.txt"),
												file_system.pathname (a_log_dirname, "proxy_log.txt"))
			end
		end

feature {NONE} -- Implementation

	generate_ace_file (a_pathname: STRING) is
			-- Generate ace file.
		require
			a_pathname_not_void: a_pathname /= Void
			a_pathname_not_empty: not a_pathname.is_empty
		local
			ace_file: KL_TEXT_OUTPUT_FILE
			file_name: STRING
		do
			file_name := file_system.pathname (a_pathname, ace_filename)
			if not file_system.file_exists (file_name) then
				create ace_file.make (file_name)
				ace_file.open_write
				if ace_file.is_open_write then
					ace_file.put_string ("system%N")
					ace_file.put_string ("%T%"" + interpreter_system_name + "%"%N%N")
					ace_file.put_string ("root%N")
					ace_file.put_string ("%TITP_INTERPRETER: execute%N%N")
					ace_file.put_string ("default%N")
					ace_file.put_string ("%Tassertion (all)%N")

					ace_file.put_line ("%Tworking_directory (%"" + file_system.absolute_pathname (a_pathname) + "%")")
					ace_file.put_line ("%Tconsole_application (yes)")
					ace_file.put_line ("%Tsyntax_warning (no)")
					ace_file.put_line ("cluster")
					ace_file.put_line ("%Treflection_library:%"" + file_system.absolute_pathname (file_system.pathname (a_pathname, "reflection_library")) + "%"")
					if is_debug_mode_enabled then
						generate_no_assertion_override (ace_file)
					end
					ace_file.put_line ("%Tall runtime:%"" + file_system.absolute_pathname (file_system.pathname (a_pathname, "runtime")) + "%"")
					ace_file.put_line ("%T%Texclude")
					ace_file.put_line ("%T%T%T%"empty_universe%";")
					if is_debug_mode_enabled then
						generate_no_assertion_override (ace_file)
					else
						ace_file.put_line ("%T%Tend")
					end
					generate_clusters_clause (ace_file, universe.clusters)
					ace_file.put_new_line

					ace_file.put_line ("external")
					if external_text /= Void then
						ace_file.put_line (external_text)
					end
					ace_file.put_string ("end")
					ace_file.close
				end
			end
		end

	generate_no_assertion_override (an_output_file: KL_TEXT_OUTPUT_FILE) is
			-- Generate clause that overides the assertion checking level to none for a cluster.
			-- TODO: This does not work as expected at the moment. Research exactly what assertions to turn off.
		require
			an_output_file_not_void: an_output_file /= Void
		do
			an_output_file.put_line ("%T%T%Tdefault")
			an_output_file.put_line ("%T%T%T%Tassertion (no)")
			an_output_file.put_line ("%T%T%Tend")
		end

	generate_clusters_clause (an_output_file: KL_TEXT_OUTPUT_FILE;
								a_clusters: ET_CLUSTERS) is
			-- Generate cluster section from `a_clusters' to
			-- `an_output_file'. Relative clusters in `a_clusters' will
			-- be taken relative from `a_base_directory'.
		require
			an_output_file_not_void: an_output_file /= Void
			an_output_file_is_open_write: an_output_file.is_open_write
			a_cluster_not_void: a_clusters /= Void
		local
			cs: DS_LINEAR_CURSOR [ET_CLUSTER]
			pathname: STRING
		do
			from
				cs := a_clusters.clusters.new_cursor
				cs.start
			until
				cs.off
			loop
				if not cs.item.is_abstract then
					an_output_file.put_character ('%T')
					an_output_file.put_string (cs.item.full_name ('_'))
					an_output_file.put_string (": %"")
					pathname := file_system.pathname_from_file_system (cs.item.full_pathname, unix_file_system)
					an_output_file.put_string (file_system_routines.interpreted_pathname (base_dirname, pathname))
					an_output_file.put_string ("%"%N")
				end
				if cs.item.subclusters /= Void then
					generate_clusters_clause (an_output_file, cs.item.subclusters)
				end
				cs.forth
			end
		end

	ace_filename: STRING is "interpreter.ace"
			-- File name of interpreter ace file

	interpreter_system_name: STRING is "interpreter"
			-- Name of interpreter system

	interpreter_filename: STRING is
			-- File name of interpreter executable
		once
			create Result.make (interpreter_system_name.count + file_system.exe_extension.count)
			Result.append_string (interpreter_system_name)
			Result.append_string (file_system.exe_extension)
		ensure
			name_not_void: Result /= Void
			name_not_empty: Result.count > 0
		end

end
