indexing

	description:

		"Error Handler for AutoTest"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUT_ERROR_HANDLER

inherit

	ERL_G_ERROR_HANDLER
		redefine
			Usage_message,
			Version_message,
			Version_number
		end

create

	make

feature -- Reporting info

	report_ise_eiffel_output (a_message: STRING) is
			-- Report the message `a_string' from the ISE Eiffel compiler
		require
			a_message_not_void: a_message /= Void
		do
			report_info_message (a_message)
		end

	report_testing_finished (a_file: STRING) is
			-- Report that testing fininshed and that the results can be viewed by looking at file
			-- `a_file'.
		require
			a_file_not_void: a_file /= Void
		local
			text: STRING
		do
			create text.make (256)
			text.append_string ("Testing finished. To view the results direct your HTML viewer to file '")
			text.append_string (a_file)
			text.append_string ("'.")
			report_info_message (text)
		end

	report_feature_selection (a_remaining_time: DT_TIME_DURATION; a_type: ET_BASE_TYPE; a_feature: ET_FEATURE) is
			-- Report that feature `a_feature' of type `a_type' has been selected for testing.
		require
			a_remaining_time_not_void: a_remaining_time /= Void
			a_remaining_time_not_negative: a_remaining_time.second_count >= 0
			a_feature_not_void: a_feature /= Void
			a_type_not_void: a_type /= Void
		local
			text: STRING
		do
			create text.make (100)
			a_remaining_time.append_to_string (text)
			text.append_string (": selected ")
			text.append_string (a_type.to_text)
			text.append_string (".")
			text.append_string (a_feature.name.name)
			report_info_message (text)
		end

	report_no_time_for_testing (a_time_out: DT_TIME_DURATION) is
			-- Report that compilation took so long, that no time was left for testing.
		require
			a_time_out_positive: a_time_out.second_count > 0
		local
			text: STRING
		do
			create text.make (100)
			text.append_string ("No tests could be run because compiling the system under test took longer than the specified time-out (")
			text.append_integer (a_time_out.minute)
			text.append_string (" min). Maybe you should increase the time-out ('--time-out=<min>'). Note that you can reuse a previous compilation by using the command line argument '--just-test'.")
			report_info_message (text)
		end

feature -- Reporting errors

	report_invalid_time_out_value (a_value: STRING) is
			-- Report that `a_value' is not a correct time-out value.
		require
			a_value_not_void: a_value /= Void
		local
			an_error: AUT_INVALID_TIME_OUT_VALUE_ERROR
		do
			create an_error.make (a_value)
			report_error (an_error)
		end

	report_unknown_class_error (a_class_name: STRING) is
			-- Report that class named `a_class_name' cannot be tested because it is unknown.
		require
			a_class_name_not_void: a_class_name /= Void
		local
			a_text: STRING
		do
			a_text := "Class " + a_class_name + " cannot be tested because it is unknown."
			report_error_message (a_text)
		end

	report_erroneous_class_error (a_class_name: STRING) is
			-- Report that class named `a_class_name' cannot be tested because it contains errors.
		require
			a_class_name_not_void: a_class_name /= Void
		local
			a_text: STRING
		do
			a_text := "Class " + a_class_name + " cannot be tested because it contains errors."
			report_error_message (a_text)
		end

	report_expanded_types_not_yet_supported (a_class_type: ET_CLASS_TYPE) is
			-- Report that the expanded classes are not yet supported, but `a_class_type' is requested to be tested.
		require
			a_class_type_not_void: a_class_type /= Void
			a_class_type_is_expanded: a_class_type.is_expanded
		do
			report_error_message ("Expanded types cannot be tested yet, but " + a_class_type.to_text + " has been requested to be tested.")
		end

	report_no_classes_specified_error is
			-- Report that no classes have been specified.
		do
			report_error_message ("No classes have been specified for testing. Please append a list of classes to test as command line arguments.")
		end

	report_interpreter_generation_error is
			-- Report that the interpreter could not be generated.
		do
			report_error_message ("Could not generate interpreter.")
		end

	report_html_generation_error is
			-- Report that an error occured whil generation the html pages
		do
			report_error_message ("Could not generate HTML pages.")
		end

feature

	Usage_message: UT_USAGE_MESSAGE is
			-- Erl-G usage message
		once
			create Result.make
			("%T[--help] [--version] [--verbose] [--just-test] [--define=<...>] [--output-dir=<...>] [--time-out=<min>] ace_filename class_name+")
		end

	Version_message: UT_MESSAGE is
			-- 'Version' message
		do
			create Result.make ("auto_test version " + Version_number)
		end

	Version_number: STRING is
			-- Current version number of AutoTest
		once
			Result := "1.0.0"
		end

end
