indexing
	description:

		"Response handler that does nothing"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUT_DEFAULT_INTERPRETER_RESPONSE_HANDLER

inherit

	AUT_INTERPRETER_RESPONSE_HANDLER
		rename
			make as make_response_handler
		end

	EXCEP_CONST
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make (a_universe: like universe; a_queue: like queue; an_interpreter: like interpreter) is
			-- Create new handler.
		require
			a_universe_not_void: a_universe /= Void
			a_queue_not_void: a_queue /= Void
			an_interpreter_not_void: an_interpreter /= Void
		do
			universe := a_universe
			queue := a_queue
			interpreter := an_interpreter
			create result_repository.make
		ensure
			universe_set: universe = a_universe
			queue_set: queue = a_queue
			interpreter_set: interpreter = an_interpreter
		end

feature -- Access

	queue: AUT_DYNAMIC_PRIORITY_QUEUE
			-- Priority queue used to mark calls

	interpreter: AUT_INTERPRETER_PROXY
			-- Interpreter

	result_repository: AUT_TEST_CASE_RESULT_REPOSITORY
			-- Result repository

feature -- Reporting

	report_object_creation_response (a_response: AUT_INTERPRETER_RESPONSE; a_receiver: ITP_VARIABLE; a_type: ET_BASE_TYPE; a_procedure: ET_PROCEDURE; an_argument_list: DS_LINEAR [ITP_EXPRESSION]) is
		do
			mark_feature_of_type (a_procedure, a_type)
			judge_invocation (a_response, a_procedure, a_type.direct_base_class (universe))
		end

	report_default_object_creation_response (a_response: AUT_INTERPRETER_RESPONSE; a_receiver: ITP_VARIABLE; a_type: ET_BASE_TYPE) is
		local
			procedure: ET_PROCEDURE
		do
			procedure := a_type.direct_base_class (universe).seeded_procedure (universe.default_create_seed)
			check
				procedure_not_void: procedure /= Void
			end
			mark_feature_of_type (procedure, a_type)
			judge_invocation (a_response, procedure, a_type.direct_base_class (universe))
		end

	report_feature_invocation_response (a_response: AUT_INTERPRETER_RESPONSE; a_feature: ET_FEATURE; a_target: ITP_VARIABLE; a_target_type: ET_BASE_TYPE; an_argument_list: DS_LINEAR [ITP_EXPRESSION]) is
		do
			mark_feature_of_type (a_feature, a_target_type)
			judge_invocation (a_response, a_feature, a_target_type.direct_base_class (universe))
		end

	report_feature_invocation_and_assignment_response (a_response: AUT_INTERPRETER_RESPONSE; a_receiver: ITP_VARIABLE; a_query: ET_QUERY; a_target: ITP_VARIABLE; a_target_type: ET_BASE_TYPE; an_argument_list: DS_LINEAR [ITP_EXPRESSION]) is
		do
			mark_feature_of_type (a_query, a_target_type)
			judge_invocation (a_response, a_query, a_target_type.direct_base_class (universe))
		end

	report_expression_assignment_response (a_response: AUT_INTERPRETER_RESPONSE; a_receiver: ITP_VARIABLE; an_expression: ITP_EXPRESSION) is
		do
			-- Nothing to do.
		end

feature {NONE}  -- Implementation

	mark_feature_of_type (a_feature: ET_FEATURE; a_type: ET_BASE_TYPE) is
			-- Mark feature `a_feature' of type `a_type'.
		require
			a_feature_not_void: a_feature /= Void
			a_type_not_void: a_type /= Void
		local
			feature_of_type: AUT_FEATURE_OF_TYPE
		do
			create feature_of_type.make (a_feature, a_type)
			queue.mark (feature_of_type)
		end

	judge_invocation (a_response: AUT_INTERPRETER_RESPONSE; a_feature: ET_FEATURE; a_class: ET_CLASS) is
			-- Judge invocation of `a_feature' of class `a_class' which triggered the response `a_response'.
			-- Note that also creations can be judged with this routine.
		require
			a_response_not_void: a_response /= Void
			a_feature_not_void: a_feature /= Void
			a_class_not_void: a_class /= Void
		local
			test_case_result: AUT_TEST_CASE_RESULT
		do
			if a_response.was_success then
				create test_case_result.make_pass (a_class, a_feature, a_response)
			elseif a_response.was_bad then
				create test_case_result.make_bad_response (a_class, a_feature, a_response)
			elseif a_response.was_void_target_call then
				create test_case_result.make_invalid (a_class, a_feature, a_response)
			elseif a_response.was_exception_thrown then
				if a_response.exception_code = Precondition and a_response.trace_depth = 1 then
					create test_case_result.make_invalid (a_class, a_feature, a_response)
				else
					-- TODO: Should we also blame someone else?
					create test_case_result.make_fail (a_class, a_feature, a_response)
				end
			else
				check
					dead_end: False
				end
			end
			add_test_case_result (test_case_result)
		end

	add_test_case_result (a_result: AUT_TEST_CASE_RESULT) is
			-- Adds test case result `a_result' to the global statistics.
		require
			a_result_not_void: a_result /= Void
		do
			result_repository.add_result (a_result)
		end

invariant

	queue_not_void: queue /= Void
	interpreter_not_void: interpreter /= Void

end