indexing
	description:

		"Response from interpreter after a request"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUT_INTERPRETER_RESPONSE

inherit

	EXCEP_CONST
		export {NONE} all end

create

	make_bad,
	make_exception,
	make_success,
	make_void_target

feature {NONE} -- Initialization

	make_bad (a_request_log: like request_log; a_response_log: like response_log) is
			-- Create new response, indicating a bad response from the interpreter.
		require
			a_request_log_not_void: a_request_log /= Void
			a_response_log_not_void: a_response_log /= Void
		do
			was_bad := True
			request_log := a_request_log
			response_log := a_response_log
		ensure
			was_bad: was_bad
			request_log_set: request_log = a_request_log
			response_log_set: response_log = a_response_log
		end

	make_exception (a_request_log: like request_log; a_response_log: like response_log; an_exception_code: like exception_code; an_exception_recipient_name: like exception_recipient_name;
					an_exception_class_name: like exception_class_name; an_exception_tag_name: like exception_tag_name;
					an_exception_trace: like exception_trace) is
			-- Create new response, indicating that an exception was thrown.
		require
			a_request_log_not_void: a_request_log /= Void
			a_response_log_not_void: a_response_log /= Void
			an_exception_recipient_name_not_void: an_exception_recipient_name /= Void
			an_exception_class_name_not_void: an_exception_class_name /= Void
			an_exception_tag_name_not_void: an_exception_tag_name /= Void
			an_exception_trace_not_void: an_exception_trace /= Void
		do
			was_exception_thrown := True
			request_log := a_request_log
			response_log := a_response_log
			exception_code := an_exception_code
			exception_recipient_name := an_exception_recipient_name
			exception_class_name := an_exception_class_name
			exception_tag_name := an_exception_tag_name
			exception_trace := an_exception_trace
		ensure
			was_exception: was_exception_thrown
			request_log_set: request_log = a_request_log
			response_log_set: response_log = a_response_log
			exception_code_set: exception_code = an_exception_code
			exception_recipient_name_set: exception_recipient_name = an_exception_recipient_name
			exception_class_name_set: exception_class_name = an_exception_class_name
			exception_tag_name_set: exception_tag_name = an_exception_tag_name
			exception_trace_set: exception_trace = an_exception_trace
		end

	make_success (a_request_log: like request_log; a_response_log: like response_log) is
			-- Create new response, indicating a successfull call.
		require
			a_request_log_not_void: a_request_log /= Void
			a_response_log_not_void: a_response_log /= Void
		do
			was_success := True
			request_log := a_request_log
			response_log := a_response_log
		ensure
			was_success: was_success
			request_log_set: request_log = a_request_log
			response_log_set: response_log = a_response_log
		end

	make_void_target is
			-- Create new response, indicating a feature call on a `Void' target.
		do
			was_void_target_call := True
			create request_log.make (0)
			create response_log.make (0)
		ensure
			was_void_target_call: was_void_target_call
			request_log_empty: request_log.is_equal ("")
			response_log_empty: response_log.is_equal ("")
		end

feature -- Status

	was_exception_thrown: BOOLEAN
			-- Has an exception been thrown?

	was_bad: BOOLEAN
			-- Was it a bad (unexpected) response?

	was_success: BOOLEAN
			-- Was the request executed successfully?

	was_void_target_call: BOOLEAN
			-- Was it a feature call with a `Void' target?

feature -- Access

	response_log: STRING
			-- Textual response of interpreter

	request_log: STRING
			-- Textual request for interpreter

	exception_code: INTEGER
			-- Code of last exception

	exception_recipient_name: STRING
			-- Name of exception recipient

	exception_class_name: STRING
			-- Name of exception receiver's class name

	exception_tag_name: STRING
			-- Name of exception tag

	exception_trace: STRING
			-- Stack trace of exception

	exception_name: STRING is
			-- Name of `exception'
		require
			was_exception_thrown
		do
			if exception_code = Void_call_target then
				Result := "Void_call_target"
			elseif exception_code = No_more_memory then
				Result := "No_more_memory"
			elseif exception_code = Precondition then
				Result := "Precondition"
			elseif exception_code = Postcondition then
				Result := "Postcondition"
			elseif exception_code = Floating_point_exception then
				Result := "Floating_point_exception"
			elseif exception_code = Class_invariant then
				Result := "Class_invariant"
			elseif exception_code = Check_instruction then
				Result := "Check_instruction"
			elseif exception_code = Routine_failure then
				Result := "Routine_failure"
			elseif exception_code = Incorrect_inspect_value then
				Result := "Incorrect_inspect_value"
			elseif exception_code = Loop_variant then
				Result := "Loop_variant"
			elseif exception_code = Loop_invariant then
				Result := "Loop_invariant"
			elseif exception_code = Signal_exception then
				Result := "Signal_exception"
			elseif exception_code = Rescue_exception then
				Result := "Rescue_exception"
			elseif exception_code = External_exception then
				Result := "External_exception"
			elseif exception_code = Void_assigned_to_expanded then
				Result := "Void_assigned_to_expanded"
			elseif exception_code = Io_exception then
				Result := "Io_exception"
			elseif exception_code = Operating_system_exception then
				Result := "Operating_system_exception"
			elseif exception_code = Retrieve_exception then
				Result := "Retrieve_exception"
			elseif exception_code = Developer_exception then
				Result := "Developer_exception"
			elseif exception_code = Runtime_io_exception then
				Result := "Runtime_io_exception"
			elseif exception_code = Com_exception then
				Result := "Com_exception"
			elseif exception_code = Runtime_check_exception then
				Result := "Runtime_check_exception"
			else
					check
						dead_end: False
					end
			end
		end

	trace_depth: INTEGER is
			-- Depth of exception trace stored in `a_string' (without interpreter frames)
		require
			was_exception_thrown: was_exception_thrown
		local
			splitter: ST_SPLITTER
			list: DS_LIST [STRING]
			cs: DS_LIST_CURSOR [STRING]
			erl_frame_found: BOOLEAN
		do
			create splitter.make
			splitter.set_separators ("%N")
			list := splitter.split (exception_trace)
				check
					list_count_greater_4: list.count >= 5
				end
			from
				cs := list.new_cursor
				cs.start
				go_after_next_dash_line (cs)
				go_after_next_dash_line (cs)
				Result := 0
			until
				cs.off or erl_frame_found
			loop
				if
					cs.item.count > erl_type_imp.count and then
					(cs.item.substring (1, erl_type_imp.count).is_equal (erl_type_imp) or
					 cs.item.substring (1, erl_type_imp.count).is_equal (erl_function_imp))
				then
					erl_frame_found := True
				else
					Result := Result + 1
					go_after_next_dash_line (cs)
				end
			end
			cs.go_after
		ensure
			result_positive: Result >= 0
		end

feature {NONE} -- Implementation

	go_after_next_dash_line (a_cs: DS_LINEAR_CURSOR [STRING]) is
		require
			a_cs_not_void: a_cs /= Void
			a_cs_not_off: not a_cs.off
		local
			dash_line_found: BOOLEAN
		do
			from
			until
				dash_line_found or a_cs.off
			loop
				if a_cs.item.is_equal (dash_line) then
					dash_line_found := True
				end
				a_cs.forth
			end
		end

	erl_type_imp: STRING is "ERL_TYPE_IMP_"
	erl_function_imp: STRING is "ERL_FUNCTION_IMP"
	dash_line: STRING is "-------------------------------------------------------------------------------"

invariant

	exclusivity: was_exception_thrown xor was_bad xor was_success xor was_void_target_call
	response_log_not_void: response_log /= Void
	request_log_not_void: request_log /= Void
	exception_recipient_not_void: was_exception_thrown = (exception_recipient_name /= Void)
	exception_class_name_not_void: was_exception_thrown = (exception_class_name /= Void)
	exception_tag_name_not_void: was_exception_thrown = (exception_tag_name /= Void)
	exception_trace_not_void: was_exception_thrown = (exception_trace /= Void)

end