indexing
	description:

		"Handler for responses from interpreter"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

deferred class AUT_INTERPRETER_RESPONSE_HANDLER

inherit

		ERL_G_TYPE_ROUTINES

feature {NONE} -- Initialization

	make (a_universe: like universe) is
			-- Create new handler.
		require
			a_universe_not_void: a_universe /= Void
		do
			universe := a_universe
		ensure
			universe_set: universe = a_universe
		end

feature -- Access

	universe: ET_UNIVERSE
			-- Universe of interpreter

feature -- Reporting

	report_object_creation_response (a_response: AUT_INTERPRETER_RESPONSE; a_receiver: ITP_VARIABLE; a_type: ET_BASE_TYPE; a_procedure: ET_PROCEDURE; an_argument_list: DS_LINEAR [ITP_EXPRESSION]) is
			-- Report the response of an object creation request.
		require
			a_response_not_void: a_response /= Void
			a_receiver_not_void: a_receiver /= Void
			a_type_not_void: a_type /= Void
			a_procedure_not_void: a_procedure /= Void
			a_procedure_is_creation: a_type.direct_base_class (universe).is_creation_exported_to (a_procedure.name, universe.any_class, universe)
			an_argument_list_not_void: an_argument_list /= Void
			an_argument_list_doesnt_have_void: not an_argument_list.has (Void)
		deferred
		end

	report_default_object_creation_response (a_response: AUT_INTERPRETER_RESPONSE; a_receiver: ITP_VARIABLE; a_type: ET_BASE_TYPE) is
			-- Report the response of an default object creation reqeust.
		require
			a_response_not_void: a_response /= Void
			a_receiver_not_void: a_receiver /= Void
			a_type_not_void: a_type /= Void
			is_default_creatable: is_default_creatable (a_type.direct_base_class (universe), universe)
		deferred
		end

	report_feature_invocation_response (a_response: AUT_INTERPRETER_RESPONSE; a_feature: ET_FEATURE; a_target: ITP_VARIABLE; a_target_type: ET_BASE_TYPE; an_argument_list: DS_LINEAR [ITP_EXPRESSION]) is
			-- Report the response of a feature invocation.
		require
			a_response_not_void: a_response /= Void
			a_feature_not_void: a_feature /= Void
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
			an_argument_list_not_void: an_argument_list /= Void
			an_argument_list_doesnt_have_void: not an_argument_list.has (Void)
		deferred
		end

	report_feature_invocation_and_assignment_response (a_response: AUT_INTERPRETER_RESPONSE; a_receiver: ITP_VARIABLE; a_query: ET_QUERY; a_target: ITP_VARIABLE; a_target_type: ET_BASE_TYPE; an_argument_list: DS_LINEAR [ITP_EXPRESSION]) is
			-- Report the response of a query invocation where the queries result has been assigned to `a_receiver'.
		require
			a_response_not_void: a_response /= Void
			a_receiver_not_void: a_receiver /= Void
			a_query_not_void: a_query /= Void
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
			an_argument_list_not_void: an_argument_list /= Void
			an_argument_list_doesnt_have_void: not an_argument_list.has (Void)
		deferred
		end

	report_expression_assignment_response (a_response: AUT_INTERPRETER_RESPONSE; a_receiver: ITP_VARIABLE; an_expression: ITP_EXPRESSION) is
			-- Report the assignement of the expressio0n `an_expression' to the variable `a_receiver'.
		require
			a_response_not_void: a_response /= Void
			a_receiver_not_void: a_receiver /= Void
			a_constant_not_void: an_expression /= Void
		deferred
		end

invariant

	universe_not_void: universe /= Void

end