indexing

	description:

		"Generates HTML pages describing test statistics"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"


class AUT_HTML_STATISTICS_GENERATOR

inherit

	AUT_SHARED_PATHNAMES
		export {NONE} all end

	AUT_SHARED_FILE_SYSTEM_ROUTINES
		export {NONE} all end

	UT_SHARED_TEMPLATE_EXPANDER
		export {NONE} all end

	KL_SHARED_FILE_SYSTEM
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make (an_output_dirname: like output_dirname; a_universe: like universe; a_classes_under_test: like classes_under_test) is
			-- Create new html generator.
		require
			an_output_dirname_not_void: an_output_dirname /= Void
			a_universe_not_void: a_universe /= Void
			a_classes_under_test: a_classes_under_test /= Void
		do
			output_dirname := an_output_dirname
			universe := a_universe
			classes_under_test := a_classes_under_test
		ensure
			output_dirname_set: output_dirname = an_output_dirname
			universe_set: universe = a_universe
			classes_under_test_set: classes_under_test = a_classes_under_test
		end

feature -- Status report

	has_fatal_error: BOOLEAN
			-- Has a fatal error occured during generation?

feature -- Access

	output_dirname: STRING
			-- Directory where HTML pages are generated to

	universe: ET_UNIVERSE
			-- Universe

	absolute_index_filename: STRING is
			-- Absolute filename of main entry page
		do
			Result := file_system.pathname (output_dirname, "index.html")
		ensure
			filename_not_void: Result /= Void
		end

feature -- HTML generation

	generate (a_repository: AUT_TEST_CASE_RESULT_REPOSITORY) is
			-- Generate HTML pages describing the results from `a_repository'.
		require
			a_repository_not_void: a_repository /= Void
		local
			cs: DS_LINEAR_CURSOR [ET_CLASS]
		do
			file_system.recursive_create_directory (output_dirname)
			file_system.recursive_create_directory (file_system.pathname (output_dirname, "image"))
			copy_static_files
			create tree_content_file.make (file_system.pathname (output_dirname, "treeContent.js"))
			tree_content_file.open_write
			generate_tree_content_file_header
			generate_summary_page (a_repository)
			generate_header_page
			if not tree_content_file.is_open_write then
				has_fatal_error := True
			else
				from
					cs := a_repository.classes.new_cursor
					cs.start
				until
					cs.off
				loop
					generate_class (cs.item, a_repository)
					cs.forth
				end
			end
			generate_tree_content_file_footer
			tree_content_file.close
		end

feature {NONE} -- HTML Generation

	copy_static_files is
			-- Copy static files from test studio directory to html directory.
		do
			file_system_routines.copy_recursive (pathnames.image_dirname, file_system.pathname (output_dirname, "image"))
			file_system_routines.copy_recursive (pathnames.misc_html_dirname, output_dirname)
		end

	generate_summary_page (a_repository: AUT_TEST_CASE_RESULT_REPOSITORY) is
			-- Generate main summary page.
		require
			a_repository_not_void: a_repository /= Void
		local
			file: KL_TEXT_OUTPUT_FILE
			set: AUT_TEST_CASE_RESULT_SET
		do
			create file.make (file_system.pathname (output_dirname, "summary.html"))
			file.open_write
			if file.is_open_write then
				set := a_repository.results
				file.put_string (template_expander.expand_from_array (summary_page_header_template, <<universe.system_name>>))
				generate_summary (set, file)
				file.put_string (summary_page_footer_template)
				file.close
			else
				has_fatal_error := True
			end
		end

	generate_header_page is
			-- Generate header page.
		local
			file: KL_TEXT_OUTPUT_FILE
		do
			create file.make (file_system.pathname (output_dirname, "header.html"))
			file.open_write
			if file.is_open_write then
				file.put_string (template_expander.expand_from_array (header_page_template, <<universe.system_name>>))
				file.close
			else
				has_fatal_error := True
			end
		end

	generate_class (a_class: ET_CLASS; a_repository: AUT_TEST_CASE_RESULT_REPOSITORY) is
			-- Generate HTML pages describing the results from `a_repository' of class `a_class'.
		require
			a_repository_not_void: a_repository /= Void
			a_class_not_void: a_class /= Void
			tree_content_file_not_void: tree_content_file /= Void
			tree_content_file_open_write: tree_content_file.is_open_write
		local
			i: INTEGER
			list: ET_FEATURE_LIST
			count: INTEGER
			file: KL_TEXT_OUTPUT_FILE
			absolute_filename: STRING
			filename: STRING
			class_node_id: INTEGER
			set: AUT_TEST_CASE_RESULT_SET
			image_filename: STRING
		do
			filename := class_summary_file_name (a_class)
			absolute_filename := file_system.pathname (output_dirname, filename)
			last_node_id := last_node_id + 1
			class_node_id := last_node_id
			set := a_repository.results_by_class (a_class)

			create image_filename.make (100)
			image_filename.append_string ("image/class_")
			if set.is_bad_response then
				image_filename.append_string ("bad_response")
			elseif set.is_fail then
				image_filename.append_string ("fail")
			elseif set.is_invalid then
				image_filename.append_string ("invalid")
			elseif set.is_pass then
				image_filename.append_string ("pass")
			elseif set.is_untested then
				image_filename.append_string ("untested")
			else
				check
					dead_end: False
				end
			end
			image_filename.append_string (".gif")
			tree_content_file.put_string ("%TaTree.add (")
			tree_content_file.put_integer (class_node_id)
			tree_content_file.put_string (", ")
			if is_class_in_test_scope (a_class) then
				tree_content_file.put_integer (test_scope_node_id)
			else
				tree_content_file.put_integer (other_node_id)
			end
			tree_content_file.put_string (", '")
			tree_content_file.put_string (a_class.name.name)
			tree_content_file.put_string ("', '")
			tree_content_file.put_string (filename)
			tree_content_file.put_string ("', '', '', '")
			tree_content_file.put_string (image_filename)
			tree_content_file.put_string ("', '")
			tree_content_file.put_string (image_filename)
			tree_content_file.put_string ("');")

			create file.make (absolute_filename)
			file.open_write
			if file.is_open_write then
				file.put_string (template_expander.expand_from_array (class_summary_header_template, <<a_class.name.name>>))
				if not a_class.implementation_checked or else a_class.has_implementation_error then
					file.put_line (class_has_error_template)
				end
				generate_summary (set, file)
				file.put_string (class_summary_footer_template)
				file.close
				from
					i := 1
					list := a_class.queries
					count := list.count
				until
					i > count
				loop
					if not list.item (i).name.is_prefix and not list.item (i).name.is_infix then
						generate_feature (a_class, list.item (i), class_node_id, a_repository)
					end
					i := i + 1
				end
				from
					i := 1
					list := a_class.procedures
					count := list.count
				until
					i > count
				loop
					if not list.item (i).name.is_prefix and not list.item (i).name.is_infix then
						generate_feature (a_class, list.item (i), class_node_id, a_repository)
					end
					i := i + 1
				end
			else
				has_fatal_error := True
			end
		end

	generate_feature (a_class: ET_CLASS; a_feature: ET_FEATURE; a_class_node_id: INTEGER; a_repository: AUT_TEST_CASE_RESULT_REPOSITORY) is
			-- Generate HTML pages describing the results from `a_repository' of class `a_class'.
		require
			a_repository_not_void: a_repository /= Void
			a_class_not_void: a_class /= Void
			a_feature_not_void: a_feature /= Void
			a_feature_not_prefix_and_not_infix: not a_feature.name.is_prefix and not a_feature.name.is_infix
			tree_content_file_not_void: tree_content_file /= Void
			tree_content_file_open_write: tree_content_file.is_open_write
		local
			file: KL_TEXT_OUTPUT_FILE
			set: AUT_TEST_CASE_RESULT_SET
			cs: DS_LINEAR_CURSOR [AUT_TEST_CASE_RESULT]
			absolute_filename: STRING
			filename: STRING
			feature_node_id: INTEGER
			i: INTEGER
			explain_link: STRING
			status: STRING
		do
			filename := feature_summary_file_name (a_class, a_feature)
			absolute_filename := file_system.pathname (output_dirname, filename)
			last_node_id := last_node_id + 1
			feature_node_id := last_node_id
			set := a_repository.results_by_feature_and_class (a_feature, a_class)

			tree_content_file.put_string ("%TaTree.add (")
			tree_content_file.put_integer (feature_node_id)
			tree_content_file.put_string (", ")
			tree_content_file.put_integer (a_class_node_id)
			tree_content_file.put_string (", '")
			tree_content_file.put_string (a_feature.name.name)
			tree_content_file.put_string ("', '")
			tree_content_file.put_string (filename)
			tree_content_file.put_string ("', '', '', 'image/feature_")
			if set.is_bad_response then
				tree_content_file.put_string ("bad_response")
			elseif set.is_fail then
				tree_content_file.put_string ("fail")
			elseif set.is_invalid then
				tree_content_file.put_string ("invalid")
			elseif set.is_pass then
				tree_content_file.put_string ("pass")
			elseif set.is_untested then
				tree_content_file.put_string ("untested")
			else
				check
					dead_end: False
				end
			end
			tree_content_file.put_line (".gif');")

			create file.make (absolute_filename)
			file.open_write
			if file.is_open_write then
				file.put_string (template_expander.expand_from_array (feature_summary_header_template,
																		<<a_class.name.name,
																		a_feature.name.name>>))
				generate_summary (set, file)
				file.put_line ("<h1>Individual Test Cases:</h1>")
				from

					cs := set.list.new_cursor
					cs.start
					i := 1
				until
					cs.off
				loop
					file.put_string ("<h2>Test Case ")
					file.put_integer (i)
					file.put_line ("</h2>")
					if cs.item.is_pass then
						explain_link := "test_case_pass.html"
						status := "pass"
					elseif cs.item.is_fail then
						explain_link := "test_case_fail.html"
						status := "fail"
					elseif cs.item.is_invalid then
						explain_link := "test_case_invalid.html"
						status := "invalid"
					elseif cs.item.is_bad_response then
						explain_link := "test_case_bad_response.html"
						status := "bad response"
					else
						check
							dead_end: False
						end
					end
					file.put_line (template_expander.expand_from_array (explain_link_template, <<explain_link, status>>))
					file.put_line ("<pre>")
					file.put_string (cs.item.response.request_log)
					file.put_new_line
					file.put_string (cs.item.response.response_log)
					file.put_line ("</pre>")
					file.put_line ("<br/>")
					cs.forth
					i := i + 1
				end
				file.put_string (feature_summary_footer_template)
				file.close
			else
				has_fatal_error := True
			end
		end

	generate_summary (a_set: AUT_TEST_CASE_RESULT_SET; a_stream: KI_TEXT_OUTPUT_STREAM) is
			-- Generate summary for set `a_set' into `a_stream'.
		require
			a_set_not_void: a_set /= Void
			a_stream_not_void: a_stream /= Void
			a_stream_open_write: a_stream.is_open_write
		local
			explain_link: STRING
			status: STRING
		do
				a_stream.put_string ("<h2>Status: <b>")
				if a_set.is_bad_response then
					explain_link := "test_case_set_bad_response.html"
					status := "bad response"
				elseif a_set.is_fail then
					explain_link := "test_case_set_fail.html"
					status := "fail"
				elseif a_set.is_invalid then
					explain_link := "test_case_set_invalid.html"
					status := "invalid"
				elseif a_set.is_pass then
					explain_link := "test_case_set_pass.html"
					status := "pass"
				elseif a_set.is_untested then
					explain_link := "test_case_set_untested.html"
					status := "untested"
				else
					check
						dead_end: False
					end
				end
				a_stream.put_line (template_expander.expand_from_array (explain_link_template, <<explain_link, status>>))
				a_stream.put_line ("</b></h2>")
				a_stream.put_string (template_expander.expand_from_array (summary_table_template,
					<<a_set.total_count.out,
						a_set.bad_response_count.out,
						a_set.fail_count.out,
						a_set.invalid_count.out,
						a_set.pass_count.out
						>>))
		end

	generate_tree_content_file_header is
			-- Generate header for `tree_file'.
		require
			tree_content_file_not_void: tree_content_file /= Void
			tree_content_file_open_write: tree_content_file.is_open_write
			last_node_id_is_zero: last_node_id = 0
		do
			test_scope_node_id := 1
			other_node_id := 2
			last_node_id := 2

			tree_content_file.put_line ("function fillTree (aTree)")
			tree_content_file.put_line ("{")
			tree_content_file.put_line ("%Td.config.target = %"content%";")
			tree_content_file.put_line ("%Td.config.inOrder = true;")
			tree_content_file.put_string ("%Td.add (0, -1, '")
			tree_content_file.put_string (universe.system_name)
			tree_content_file.put_line ("', 'summary.html');")
			tree_content_file.put_line ("%Td.add (1, 0, 'Classes selected for testing', 'summary.html');")
			tree_content_file.put_line ("%Td.add (2, 0, 'Other classes', 'summary.html');")
		end

	generate_tree_content_file_footer is
			-- Generate header for `tree_file'.
		require
			tree_content_file_not_void: tree_content_file /= Void
			tree_content_file_open_write: tree_content_file.is_open_write
		do
			tree_content_file.put_line ("}")
		end

feature {NONE} -- File handles

	tree_content_file: KL_TEXT_OUTPUT_FILE
			-- File that contains script to build Javascript tree

feature {NONE} -- Implementation

	class_summary_file_name (a_class: ET_CLASS): STRING is
				-- File name for class sumamry file for class `a_class'
			require
				a_class_not_void: a_class /= Void
			do
				create Result.make (a_class.name.name.count + class_summary_file_postfix.count)
				Result.append_string (a_class.name.name)
				Result.to_lower
				Result.append_string (class_summary_file_postfix)
			ensure
				name_not_void: Result /= Void
				name_not_empty: Result.count > 0
			end

	feature_summary_file_name (a_class: ET_CLASS; a_feature: ET_FEATURE): STRING is
				-- File name for feature sumamry file for feature `a_feature' of class `a_class'
			require
				a_class_not_void: a_class /= Void
				a_feature_not_void: a_feature /= Void
			do
				create Result.make (a_class.name.name.count + 1 + a_feature.name.name.count + feature_summary_file_postfix.count)
				Result.append_string (a_class.name.name)
				Result.to_lower
				Result.append_character ('-')
				Result.append_string (a_feature.name.name)
				Result.append_string (feature_summary_file_postfix)
			ensure
				name_not_void: Result /= Void
				name_not_empty: Result.count > 0
			end

	last_node_id: INTEGER
			-- Last node id uses

	test_scope_node_id: INTEGER
			-- Id of node being parent to all classes under test

	other_node_id: INTEGER
			-- Id of node being parent to all classes that were not under test


feature {NONE} -- Test Scope

	is_class_in_test_scope (a_class: ET_CLASS): BOOLEAN is
			-- Is class `a_class' in the test scope?
		require
			a_class_not_void: a_class /= Void
		do
			Result := classes_under_test.has (a_class)
		end

	classes_under_test: DS_LINEAR [ET_CLASS]
			-- List of classes under test

feature {NONE} -- Implementation constants

	class_summary_file_postfix: STRING is "_class_summary.html"

	feature_summary_file_postfix: STRING is "_feature_summary.html"

	class_summary_header_template: STRING is "[
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
	<head>
		<title>Class Summary</title>
		<link rel="stylesheet" href="stylesheet.css" type="text/css"/>
		<script type="text/javascript" src="explain.js"></script>
	</head>
	<body>
		<div id="content">
			<h1>${1}</h2>
]"

	class_summary_footer_template: STRING is "[
		</div>
	</body>
</html>
]"
	feature_summary_header_template: STRING is "[
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
	<head>
		<title>Feature Summary</title>
		<link rel="stylesheet" href="stylesheet.css" type="text/css"/>
		<script type="text/javascript" src="explain.js"></script>
	</head>
	<body>
		<div id="content">
			<h1>${1}.${2}</h2>
			<p>
				Extraction of short bug reproducing examples is not yet implemented.
				However, they can be extracted manually from the <a href="../log/proxy_log.txt"/> complete interpreter input and output</a>.
				To reproduce the bug, feed the whole log (or only its relevant parts) to the interpreter. The interpreter can be found in
				directory <a href="../interpreter/EIFGEN/F_code">../interpreter/EIFGEN/F_code</a>.
			</p>
]"

	feature_summary_footer_template: STRING is "[
		</div>
	</body>
</html>
]"

	summary_page_header_template: STRING is "[
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
	<head>
		<title>Summary Page</title>
		<link rel="stylesheet" href="stylesheet.css" type="text/css"/>
		<script type="text/javascript" src="explain.js"></script>
	</head>
	<body>
		<div id="content">
		<h1>System '${1}'</h1>
]"

	summary_page_footer_template: STRING is "[
		</div>
	</body>
</html>
]"
	summary_table_template: STRING is "[
			<h1>Test Case Summary</h1>
			<table>
				<tr>
					<td>Total:</td>
					<td>${1}</td>
				</tr>
				<tr>
					<td>Bad Response:</td>
					<td>${2}</td>
				</tr>
				<tr>
					<td>Fail:</td>
					<td>${3}</td>
				</tr>
				<tr>
					<td>Invalid:</td>
					<td>${4}</td>
				</tr>
				<tr>
					<td>Pass:</td>
					<td>${5}</td>
				</tr>
			</table>
]"

	header_page_template: STRING is "[
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
	<head>
		<title>AutoTest statistics for system '${1}'</title>
		<link rel="stylesheet" href="stylesheet.css" type="text/css"/>
	</head>
	<body>
		<table class="header" width="100%" border="0" cellpadding="0" cellspacing="0">
		  <tr>
			<td width="220" height="80" class="tableHeader">
				<img src="image/o_logo.png" width="152" height="80" border="0" alt="se logo">
			</td>
			<td>
				<h1 class="title">
					AutoTest statistics for system '${1}'
				</h1>
				<h2 class="title">
					Because you can!
				</h2>
			</td>
		  </tr>
		  <tr>
			<td><img src="image/spacer.png" width="1" height="10" alt=""/></td>
			<td>&nbsp;</td>
			<td>&nbsp;</td>
		  </tr>
		</table>
	</body>
</html>
]"

	class_has_error_template: STRING is "[
<ul><li>The class has syntax or type errors.</li></ul>
]"

	explain_link_template: STRING is "[
<a href="JavaScript: explain ('${1}')">${2}</a>
]"

invariant

	output_dirname_not_void: output_dirname /= Void
	universe_not_void: universe /= Void
	classes_under_test_not_void: classes_under_test /= Void

end
