indexing

	description:

		"Objects that create a list of objects"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUT_RANDOM_INPUT_CREATOR

inherit

	AUT_TASK

	AUT_SHARED_RANDOM
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make (a_universe: like universe; an_interpreter: like interpreter) is
			-- Create new feature caller.
		require
			a_universe_not_void: a_universe /= Void
			a_interpreter_not_void: an_interpreter /= Void
		do
			universe := a_universe
			interpreter := an_interpreter
			create {DS_ARRAYED_LIST [ET_BASE_TYPE]} types.make_default
			create {DS_ARRAYED_LIST [ITP_EXPRESSION]} receivers.make_default
		ensure
			universe_set: universe = a_universe
			interpreter_set: interpreter = an_interpreter
		end

feature -- Status

	has_next_step: BOOLEAN
			-- Is there a next step to execute?

feature -- Access

	object_creator: AUT_RANDOM_OBJECT_CREATOR
			-- Task that will be used to create the individual objects

	types: DS_LIST [ET_BASE_TYPE]
			-- List containing the types that the objects
			-- in `constants' will conform to;
			-- For each type one object will be created and put in
			-- the corresponding slot in `constants'

	receivers: DS_LIST [ITP_EXPRESSION]
			-- Receivers created as part of this task

	universe: ET_UNIVERSE
			-- Universe

	interpreter: AUT_INTERPRETER_PROXY
			-- Proxy to the interpreter used to execute call

feature -- Change

	add_type (a_type: ET_BASE_TYPE) is
			-- Add `a_type' to `types'.
		require
			a_type_not_void: a_type /= Void
		do
			types.force_last (a_type)
		ensure
			added: types.last = a_type
		end

	append_types (a_list: DS_LINEAR [ET_BASE_TYPE]) is
			-- Extend `types' with `a_list'.
		require
			a_list_not_void: a_list /= Void
			a_list_doesnt_have_void: not a_list.has (Void)
		do
			types.append_last (a_list)
		end

feature -- Execution

	start is
		do
			has_next_step := True
			receivers.wipe_out
			object_creator := Void
		ensure then
			no_receivers: receivers.count = 0
			object_creator_void: object_creator = Void
		end

	step is
		local
			i: INTEGER
			receiver: ITP_EXPRESSION
		do
			if object_creator /= Void then
				object_creator.step
				if not object_creator.has_next_step then
					receivers.force_last (object_creator.receiver)
					object_creator := Void
				end
			elseif receivers.count < types.count then
				random.forth
				i := (random.item  \\ 1)
				if i = 0 then
					create_object_creator
				else
					receiver := interpreter.random_conforming_variable (types.item (receivers.count + 1))
					if receiver = Void then
						create_object_creator
					end
				end
			else
				has_next_step := False
			end
		end

	cancel is
		do
			has_next_step := False
		end

feature {NONE} -- Steps

	create_object_creator is
			-- Create object creator for next receiver.
		require
			receivers.count < types.count
		do
			create object_creator.make (universe, interpreter,
											types.item (receivers.count + 1))
			object_creator.start
		end

invariant

	universe_not_void: universe /= Void
	interpreter_not_void: interpreter /= Void
	types_not_void: types /= Void
	types_doesnt_have_void: not types.has (Void)
	receivers_not_void: receivers /= Void
	receivers_doesnt_have_void: not receivers.has (Void)
	counts_valid: receivers.count <= types.count

end