indexing

	description:

		"Random testing strategy"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUT_RANDOM_STRATEGY

inherit

	AUT_STRATEGY
		rename
			make as make_strategy
		redefine
			start
		end

create

	make

feature {NONE} -- Initialization

	make (a_universe: like universe; an_interpreter: like interpreter; an_error_handler: like error_handler) is
			-- Create new strategy.
		require
			a_universe_not_void: a_universe /= Void
			a_interpreter_not_void: an_interpreter /= Void
			an_error_handler_not_void: an_error_handler /= Void
		do
			make_strategy (a_universe, an_interpreter)
			create queue.make (universe)
			error_handler := an_error_handler
		ensure
			universe_set: universe = a_universe
			interpreter_set: interpreter = an_interpreter
			error_handler_set: error_handler = an_error_handler
		end

feature -- Status

	has_next_step: BOOLEAN is
		do
			Result := (sub_task /= Void and then sub_task.has_next_step) or (queue.highest_dynamic_priority > 0)
		end

feature -- Access

	queue: AUT_DYNAMIC_PRIORITY_QUEUE
			-- Feature queue

	result_repository: AUT_TEST_CASE_RESULT_REPOSITORY
			-- Result repository

	error_handler: AUT_ERROR_HANDLER
			-- Error handler

	selected_feature: AUT_FEATURE_OF_TYPE
			-- Currently selected feature;
			-- `Void' if none is selected.

feature -- Execution

	start is
		local
			response_handler: AUT_DEFAULT_INTERPRETER_RESPONSE_HANDLER
		do
			Precursor
			create response_handler.make (universe, queue, interpreter)
			interpreter.set_response_handler (response_handler)
			result_repository := response_handler.result_repository
		ensure then
			result_repository_not_void: result_repository /= Void
		end

	step is
		local
			caller: AUT_RANDOM_FEATURE_CALLER
		do
			if sub_task = Void then
				queue.select_next
				selected_feature := queue.last_feature
				create caller.make (universe, interpreter)
				caller.set_feature_and_type (selected_feature.feature_, selected_feature.type)
				sub_task := caller
				sub_task.start
			else
				if sub_task.has_next_step and interpreter.is_running and interpreter.is_ready then
					sub_task.step
				else
					if interpreter.is_running and not interpreter.is_ready then
						interpreter.stop
					end
					if not interpreter.is_running then
						interpreter.start
					end
					sub_task := Void
				end
			end
		end

feature {NONE} -- Implementation

	sub_task: AUT_TASK
			-- Current sub task

invariant

	queue_not_void: queue /= Void
	error_handler_not_void: error_handler /= Void

end
