indexing
	description: "[
		Objects that print expressions to a character output stream.
		]"
	author: "Andreas Leitner"
	date: "$Date$"
	revision: "$Revision$"

class AUT_EXPRESSION_PRINTER

inherit

	ITP_EXPRESSION_PROCESSOR

	UT_CHARACTER_FORMATTER
		export {NONE} all end

	KL_SHARED_STREAMS
		export {NONE} all end

create

	make,
	make_null,
	make_string

feature {NONE} -- Initialization

	make (an_output_stream: like output_stream) is
			-- Create new printer using `an_output_stream' as output
			-- stream.
		require
			an_output_stream_not_void: an_output_stream /= Void
		do
			output_stream := an_output_stream
		ensure
			output_stream_set: output_stream = an_output_stream
		end

	make_null is
			-- Create new printer using the null output stream as output
			-- stream.
		do
			make (null_output_stream)
		end

	make_string (a_string: STRING) is
			-- Create a new printer appending its output
			-- to `a_string'.
		require
			a_string_not_void: a_string /= Void
		do
			create {KL_STRING_OUTPUT_STREAM} output_stream.make (a_string)
		end

feature -- Access

	output_stream: KI_CHARACTER_OUTPUT_STREAM
			-- Output stream

feature {ITP_EXPRESSION} -- Processing

	process_boolean (a_value: ITP_BOOLEAN) is
		do
			output_stream.put_boolean (a_value.value)
		end

	process_character (a_value: ITP_CHARACTER) is
		do
			put_quoted_eiffel_character (output_stream, a_value.value)
		end

	process_double (a_value: ITP_DOUBLE) is
		do
			output_stream.put_string ("{DOUBLE}")
			output_stream.put_string (a_value.value.out)
		end

	process_integer_8 (a_value: ITP_INTEGER_8) is
		do
			output_stream.put_string ("{INTEGER_8}")
			output_stream.put_integer (a_value.value)
		end

	process_integer_16 (a_value: ITP_INTEGER_16) is
		do
			output_stream.put_string ("{INTEGER_16}")
			output_stream.put_integer (a_value.value)
		end

	process_integer (a_value: ITP_INTEGER) is
		do
			output_stream.put_integer (a_value.value)
		end

	process_integer_64 (a_value: ITP_INTEGER_64) is
		do
			output_stream.put_string ("{INTEGER_64}")
			output_stream.put_string (a_value.value.out)
		end

	process_natural_8 (a_value: ITP_NATURAL_8) is
		do
			output_stream.put_string ("{NATURAL_8}")
			output_stream.put_integer (a_value.value)
		end

	process_natural_16 (a_value: ITP_NATURAL_16) is
		do
			output_stream.put_string ("{NATURAL_16}")
			output_stream.put_integer (a_value.value)
		end

	process_natural_32 (a_value: ITP_NATURAL_32) is
		do
			output_stream.put_string ("{NATURAL_32}")
			output_stream.put_string (a_value.value.out)
		end

	process_natural_64 (a_value: ITP_NATURAL_64) is
		do
			output_stream.put_string ("{NATURAL_64}")
			output_stream.put_string (a_value.value.out)
		end

	process_pointer (a_value: ITP_POINTER) is
		do
			output_stream.put_string ("itp_default_pointer")
		end

	process_reference (a_value: ITP_REFERENCE) is
		do
			if a_value.value = Void then
				output_stream.put_string ("Void")
			else
				output_stream.put_string (a_value.value.out)
			end
		end

	process_real (a_value: ITP_REAL) is
		do
			output_stream.put_string ("{REAL}")
			output_stream.put_string (a_value.value.out)
		end

	process_variable (a_value: ITP_VARIABLE) is
		do
			output_stream.put_string (a_value.name)
		end

invariant

	output_stream_not_void: output_stream /= Void

end
