%{
indexing

	description:

		"Lace parsers"

	library: "Gobo Eiffel Tools Library"
	copyright: "Copyright (c) 1999-2004, Eric Bezault and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUT_LACE_PARSER

inherit

	ET_LACE_PARSER_SKELETON

	ET_LACE_SCANNER
		rename
			make as make_lace_scanner
		end

create

	make, make_standard, make_with_factory

feature -- Access

	external_text: STRING
		-- External clause in textual format
%}

%token <ET_IDENTIFIER> L_IDENTIFIER L_STRING L_ALL
%token L_SYSTEM L_ROOT L_END L_CLUSTER
%token L_DEFAULT L_EXTERNAL L_GENERATE L_OPTION
%token L_ABSTRACT L_EXCLUDE L_VISIBLE L_LIBRARY
%token L_STRERR

%type <STRING> External_list External External_items External_terminator External_separator Identifier

%start Ace

%%
--------------------------------------------------------------------------------

Ace: L_SYSTEM Identifier L_ROOT Identifier Root_cluster_opt Creation_procedure_opt
	Defaults_opt Clusters_opt Externals_opt Generates_opt L_END
	;

Root_cluster_opt: -- Empty
	| '(' Identifier ')'
	;

Creation_procedure_opt: -- Empty
	| ':' Identifier
	;

Defaults: L_DEFAULT
	| L_DEFAULT Default_list
	;

Defaults_opt: -- Empty
	| Defaults
	;

Default_list: Default Default_terminator
	| Default Default_separator Default_list
	;

Default: Identifier '(' Identifier ')'
	| Identifier '(' L_ALL ')'
	;

Default_terminator: -- Empty
	| ';'
	;

Default_separator: -- Empty
	| ';'
	;

Clusters_opt: -- Empty
	| L_CLUSTER
	| L_CLUSTER Cluster_list Cluster_terminator
	;

Cluster_list: Cluster
	| Qualified_subcluster
	| Cluster_list Cluster_separator Cluster
	| Cluster_list Cluster_separator Qualified_subcluster
	;

Cluster: L_ABSTRACT Nested_cluster
	| L_ALL Recursive_cluster
	| L_LIBRARY Recursive_cluster
	| Nested_cluster
	;

Qualified_subcluster: Identifier '(' Identifier ')' ':' Identifier Cluster_options_opt
	| L_ALL Identifier '(' Identifier ')' ':' Identifier Cluster_options_opt
	| L_LIBRARY Identifier '(' Identifier ')' ':' Identifier Cluster_options_opt
	;

Nested_cluster: Identifier ':' Identifier Cluster_options_opt Subclusters_opt
	| Identifier Cluster_options_opt Subclusters_opt
	;

Recursive_cluster: Identifier ':' Identifier Cluster_options_opt
	;

Cluster_terminator: -- Empty
	| ';'
	;

Cluster_separator: -- Empty
	| ';'
	;

Subclusters_opt: -- Empty
	| L_CLUSTER L_END
	| L_CLUSTER Subcluster_list Cluster_terminator L_END
	;

Subcluster_list: Subcluster
	| Subcluster_list Cluster_separator Subcluster
	;

Subcluster: L_ABSTRACT Nested_cluster
	| Nested_cluster
	;

Cluster_options_opt: -- Empty
	| Excludes Defaults_opt Options_opt Visibles_opt L_END
	| Defaults Options_opt Visibles_opt L_END
	| Options Visibles_opt L_END
	| Visibles L_END
	;

Excludes: L_EXCLUDE
	| L_EXCLUDE Exclude_list
	| L_EXCLUDE Exclude_list ';'
	;

Exclude_list: Identifier
	| Exclude_list ';' Identifier
	;

Options: L_OPTION
	| L_OPTION Option_list
	;

Options_opt: -- Empty
	| Options
	;

Option_list: Option Option_terminator
	| Option Option_separator Option_list
	;

Option: Identifier '(' Identifier ')' ':' Class_list
	| Identifier '(' L_ALL ')' ':' Class_list
--TODO: | Identifier ':' Class_list
	;

Class_list: Identifier
	| Class_list ',' Identifier
	;

Option_terminator: -- Empty
	| ';'
	;

Option_separator: -- Empty
	| ';'
	;

Visibles: L_VISIBLE
	| L_VISIBLE Visible_list
	;

Visibles_opt: -- Empty
	| Visibles
	;

Visible_list: Identifier L_END
	| Visible ';'
	| Visible Visible_separator Visible_list
	;

Visible: Identifier
	| Identifier L_END
	;

Visible_separator: -- Empty
	| ';'
	;

Externals_opt: -- Empty
		{
			create external_text.make (0)
		}
	| L_EXTERNAL
		{
			external_text := ""
		}
	| L_EXTERNAL External_list
		{
			create external_text.make (1000)
			external_text.append_string ($2)
		}
	;

External_list: External External_terminator
		{
			$$ := $1 + $2
		}
	| External External_separator External_list
		{
			$$ := $1 + $2 + $3
		}
	;

External: Identifier ':' External_items
		{
			$$ := $1 + ":" + $3 + "%N"
		}
	| Identifier '(' Identifier ')'
		{
			$$ := $1 + "(" + $3 + ")"
		}
	| Identifier '(' L_ALL ')'
		{
			$$ := $1 + "(" + $3.name + ")"
		}
	;

External_items: Identifier
		{
			$$ := "%"" + $1 + "%""
		}
	| External_items ',' Identifier
		{
			$$ := $1 + ", %"" + $3 + "%""
		}
	;

External_terminator: -- Empty
		{
			$$ := ""
		}
	| ';'
		{
			$$ := ";"
		}
	;

External_separator: -- Empty
		{
			$$ := ""
		}
	| ';'
		{
			$$ := ";"
		}
	;

Generates_opt: -- Empty
	| L_GENERATE
	| L_GENERATE External_list
	;

Identifier: L_IDENTIFIER
		{ $$ := $1.name }
	| L_STRING
		{ $$ := $1.name }
	;

--------------------------------------------------------------------------------
%%

end
