indexing
	description	: "Reads line by line from a ePosix file descriptor without blocking"
	author		: "Andreas Leitner"
	date		: "$Date$"
	revision	: "1.0.0"

class AUT_NON_BLOCKING_LINE_READER

create

	make

feature {NONE} -- Initialization

	make (a_file_descriptor: like file_descriptor) is
			-- Create new line reader which reads from `a_file_descriptor'.
		require
			a_file_descriptor_not_void: a_file_descriptor /= Void
		do
			file_descriptor := a_file_descriptor
			file_descriptor.set_blocking_io (False)
			create last_line.make (256)
		ensure
			file_descriptor_set: file_descriptor = a_file_descriptor
		end

feature -- Status

	has_read_line: BOOLEAN
			-- Has a complete line been read?

	last_line: STRING
			-- Last line read; Only in a defined state when
			-- `has_read_line'.

	end_of_input: BOOLEAN is
			-- Has the end of input been reached?
		do
			Result := file_descriptor.eof
		end

feature -- Access

	file_descriptor: ABSTRACT_FILE_DESCRIPTOR
			-- File descriptor to read from

feature -- Reading

	try_read_line is
			-- Try to read a complete line. Doesn't block if it cannot
			-- read a complete line. Sets `has_read_line' to `True' if it
			-- succeeded. exit immediately if no input is available at
			-- the moment or the end of input has been reached.
		do
			if has_read_line then
				create last_line.make (256)
				has_read_line := False
			end
			if not end_of_input then
				from
					file_descriptor.read_character
				until
					file_descriptor.last_blocked or has_read_line or end_of_input
				loop
					if
						file_descriptor.last_character = '%N'
					then
						has_read_line := True
					else
						if file_descriptor.last_character /= '%R' then
							last_line.append_character (file_descriptor.last_character)
						end
						file_descriptor.read_character
					end
				end
			end
		end

invariant

	file_descriptor_not_void: file_descriptor /= Void
	has_read_line_implies_last_line: has_read_line implies last_line /= Void

end
