indexing

	description:

		"'auto_test' command line tool"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUTO_TEST

inherit

	AUTO_TEST_COMMAND_LINE_PARSER
		export {NONE} all end

	ERL_G_TYPE_ROUTINES
		export {NONE} all end

	DT_SHARED_SYSTEM_CLOCK
		export {NONE} all end

	KL_SHARED_STRING_EQUALITY_TESTER
		export {NONE} all end

	KL_SHARED_EXECUTION_ENVIRONMENT
		export {NONE} all end

	KL_SHARED_FILE_SYSTEM
		export {NONE} all end

create

	execute

feature -- Execution

	execute is
			-- Start 'erl_g' execution.
		local
			a_file: KL_TEXT_INPUT_FILE
			a_lace_parser: ET_LACE_PARSER
			an_aut_lace_parser: AUT_LACE_PARSER
			a_lace_error_handler: ET_LACE_ERROR_HANDLER
			an_xace_parser: ET_XACE_UNIVERSE_PARSER
			an_xace_error_handler: ET_XACE_DEFAULT_ERROR_HANDLER
			an_xace_variables: DS_HASH_TABLE [STRING, STRING]
			nb: INTEGER
		do
			make
			Arguments.set_program_name ("auto_test")
			output_dirname := "auto_test_gen"
			create time_out.make (0, 30, 0)
			start_time := system_clock.date_time_now
			create error_handler.make
			process_arguments
			if class_names.count = 0 then
				error_handler.report_no_classes_specified_error
				Exceptions.die (1)
			end
			create a_file.make (ace_filename)
			a_file.open_read
			if a_file.is_open_read then
				nb := ace_filename.count
				if nb > 5 and then ace_filename.substring (nb - 4, nb).is_equal (".xace") then
					create an_xace_error_handler.make_standard
					create an_xace_variables.make_map (100)
					an_xace_variables.set_key_equality_tester (string_equality_tester)
					if defined_variables /= Void then
						parse_defined_variables (defined_variables, an_xace_variables)
					end
					create an_xace_parser.make_with_variables (an_xace_variables, an_xace_error_handler)
					an_xace_parser.parse_file (a_file)
					a_file.close
					if not an_xace_error_handler.has_error then
						universe := an_xace_parser.last_universe
					end
				else
					create a_lace_error_handler.make_standard
					create a_lace_parser.make (a_lace_error_handler)
					a_lace_parser.parse_file (a_file)
					a_file.close
					if not a_lace_parser.syntax_error then
						universe := a_lace_parser.last_universe
						-- TODO: The Gobo Eiffel Lace parser does not yet provide
						-- information about the external clause. Until such support
						-- is provided the below workaround must stay.
						create an_aut_lace_parser.make_standard
						a_file.open_read
						an_aut_lace_parser.parse_file (a_file)
						a_file.close
						external_text := an_aut_lace_parser.external_text
					end
				end
				if universe /= Void then
					setup_universe
					generate_interpreter
					if interpreter /= Void then
						execute_tests
						if result_repository /= Void then
							generate_statistics
						end
					else
						error_handler.report_interpreter_generation_error
					end
				else
					Exceptions.die (3)
				end
			else
				error_handler.report_cannot_read_error (ace_filename)
				Exceptions.die (1)
			end
		end

feature -- Parsing

	parse_defined_variables (an_input: STRING; a_table: DS_HASH_TABLE [STRING, STRING]) is
			-- Parse variable definitions in `an_input' and set the variables in `a_table' accordingly.
		require
			an_input_not_void: an_input /= Void
			a_table_not_void: a_table /= Void
		local
			a_splitter: ST_SPLITTER
			a_cursor: DS_LIST_CURSOR [STRING]
			a_definition: STRING
			an_index: INTEGER
			gobo_eiffel: STRING
		do
			gobo_eiffel := Execution_environment.variable_value ("GOBO_EIFFEL")
			if gobo_eiffel /= Void then
				a_table.force_last (gobo_eiffel, "GOBO_EIFFEL")
			end
			a_table.force_last ("True", "ERL_G_NO_IMPLEMENTATION")
			create a_splitter.make
			a_cursor := a_splitter.split (defined_variables).new_cursor
			from a_cursor.start until a_cursor.after loop
				a_definition := a_cursor.item
				if a_definition.count > 0 then
					an_index := a_definition.index_of ('=', 1)
					if an_index = 0 then
						a_table.force_last ("", a_definition)
					elseif an_index = a_definition.count then
						a_table.force_last ("", a_definition.substring (1, an_index - 1))
					elseif an_index /= 1 then
						a_table.force_last (a_definition.substring (an_index + 1 ,a_definition.count), a_definition.substring (1, an_index - 1))
					end
				end
				a_cursor.forth
			end
		end

feature {NONE} -- Processing

	setup_universe is
			-- Process `universe'.
		require
			universe_not_void: universe /= Void
		local
			an_ast_factory: ET_DECORATED_AST_FACTORY
		do
			create an_ast_factory.make
			an_ast_factory.set_keep_all_breaks (True)
			universe.set_ast_factory (an_ast_factory)
			universe.error_handler.set_ise
			universe.set_use_assign_keyword (True)
			universe.set_use_attribute_keyword (False)
			universe.set_use_convert_keyword (True)
			universe.set_use_create_keyword (True)
			universe.set_use_recast_keyword (False)
			universe.set_use_reference_keyword (True)
			universe.set_ise (True)
			universe.set_use_void_keyword (True)
		end

	generate_interpreter is
			-- Generate interpreter for universe `universe'.
		require
			universe_not_empty: universe /= Void
		local
			factory: AUT_INTERPRETER_GENERATOR
			cs: DS_LINEAR_CURSOR [STRING]
			l_class: ET_CLASS
			l_class_type: ET_CLASS_TYPE
			tester: AUT_CLASS_EQUALITY_TESTER
			interpreter_base_dirname: STRING
			log_dirname: STRING
		do
			create factory.make (universe)
			factory.set_external_text (external_text)
			if is_debug_mode_enabled then
				factory.enable_debug_mode
			end
			from
				cs := class_names.new_cursor
				cs.start
				create types_under_test.make (class_names.count)
				create classes_under_test.make (class_names.count)
				create tester.make
				classes_under_test.set_equality_tester (tester)
			until
				cs.off
			loop
				l_class := universe.class_by_name (cs.item)
				if l_class = Void then
					error_handler.report_unknown_class_error (cs.item)
				elseif not l_class.implementation_checked or else l_class.has_implementation_error then
					error_handler.report_erroneous_class_error (cs.item)
					classes_under_test.force_last (l_class)
				else
					classes_under_test.force_last (l_class)
					if l_class.is_generic then
						l_class_type := generic_derivation (l_class, universe)
					else
						l_class_type := l_class
					end
					if l_class_type.is_expanded then
						error_handler.report_expanded_types_not_yet_supported (l_class_type)
					else
						factory.mark_type_reflectable (l_class_type)
						types_under_test.force_last (l_class_type)
					end
				end
				cs.forth
			end
			interpreter_base_dirname := file_system.pathname (output_dirname, "interpreter")
			log_dirname := file_system.pathname (output_dirname, "log")
			factory.set_base_dirname (file_system.dirname (ace_filename))
			factory.set_output_handler (agent error_handler.report_ise_eiffel_output)
			if not just_test then
				factory.generate_all (interpreter_base_dirname)
			end
			factory.create_interpreter (interpreter_base_dirname, log_dirname)
			interpreter := factory.last_interpreter
		ensure
			types_under_test_not_void: types_under_test /= Void
			classes_under_test_not_void: classes_under_test /= Void
		end

	execute_tests is
			-- Execute tests.
		require
			universe_not_empty: universe /= Void
			interpreter_not_void: interpreter /= Void
			types_under_test_not_void: types_under_test /= Void
		local
			cs: DS_LINEAR_CURSOR [ET_BASE_TYPE]
			strategy: AUT_RANDOM_STRATEGY
			duration: DT_TIME_DURATION
			time_now: DT_DATE_TIME
			time_left: DT_TIME_DURATION
		do
			create strategy.make (universe, interpreter, error_handler)
			from
				cs := types_under_test.new_cursor
				cs.start
			until
				cs.off
			loop
				strategy.queue.set_static_priority_of_type (cs.item, 10)
				cs.forth
			end
			time_now := system_clock.date_time_now
			duration := time_now.time_duration (start_time)
			duration.set_canonical
			time_left := time_out - duration
			time_left.set_canonical
			if time_out.second_count > 0 and time_left.second_count <= 0 then
				error_handler.report_no_time_for_testing (time_out)
			else
				from
					strategy.start
					result_repository := strategy.result_repository
				until
					(not strategy.has_next_step) or (time_out.second_count > 0 and time_left.second_count <= 0)
				loop
					if strategy.selected_feature /= Void then
						error_handler.report_feature_selection (time_left, strategy.selected_feature.type, strategy.selected_feature.feature_)
					end
					strategy.step
					time_now := system_clock.date_time_now
					duration := time_now.time_duration (start_time)
					duration.set_canonical
					time_left := time_out - duration
					time_left.set_canonical
				end
				if interpreter.is_running then
					interpreter.stop
				end
			end
		end

	generate_statistics is
			-- Generate statistics about executed tests
		require
			result_repository_not_void: result_repository /= Void
		local
			html_generator: AUT_HTML_STATISTICS_GENERATOR
		do
			create html_generator.make (file_system.pathname (output_dirname, "result"), universe, classes_under_test)
			html_generator.generate (result_repository)
			if html_generator.has_fatal_error then
				error_handler.report_html_generation_error
			else
				error_handler.report_testing_finished (html_generator.absolute_index_filename)
			end
		end

feature {NONE} -- Implementation

	universe: ET_UNIVERSE
			-- Universe under test

	interpreter: AUT_INTERPRETER_PROXY
			-- Interpreter for system under test

	types_under_test: DS_ARRAYED_LIST [ET_CLASS_TYPE]
			-- Types under test

	classes_under_test: DS_ARRAYED_LIST [ET_CLASS]
			-- Classes under test

	result_repository: AUT_TEST_CASE_RESULT_REPOSITORY
			-- Result repository

	start_time: DT_DATE_TIME
			-- Time at beginning of testing

	external_text: STRING
			-- External clause of Ace file of system under test;
			-- TODO: The Gobo Eiffel Lace parser does not yet provide
			-- information about the external clause. Until such support
			-- is provided the below workaround must stay.

end
