indexing

	description:

		"AutoTest command line parser"

	copyright: "Copyright (c) 2004-2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUTO_TEST_COMMAND_LINE_PARSER

inherit

	KL_SHARED_EXCEPTIONS
	KL_SHARED_ARGUMENTS
	GEXACE_COMMAND_LINE_PARSER

feature -- Status report

	error_handler: AUT_ERROR_HANDLER
			-- Error handler

	output_dirname: STRING
			-- Name of output directory

	defined_variables: STRING
			-- Defined variables

	ace_filename: STRING
			-- Name of ace file of input system

	class_names: DS_LINKED_LIST [STRING]
			-- List of class names to be tested

	time_out: DT_TIME_DURATION
			-- Maximal time to test;
			-- A timeout value of `0' means no time out.

	is_debug_mode_enabled: BOOLEAN
			-- Should the interpreter runtime be compiled with
			-- assertion checking on?

	just_test: BOOLEAN
			-- Should generation and compilation of the interpreter be skipped

feature -- Parsing

	process_arguments is
			-- Process arguments
		require
			error_hanlder_not_void: error_handler /= Void
		do
			if match_long_option ("help") then
				error_handler.enable_verbose
				error_handler.report_usage_message
				error_handler.disable_verbose
				Exceptions.die (0)
			end
			if match_long_option ("version") then
				error_handler.enable_verbose
				error_handler.report_version_message
				error_handler.disable_verbose
				Exceptions.die (0)
			end
			if match_long_option ("verbose") then
				error_handler.enable_verbose
				consume_option
			end

			if match_long_option ("just-test") then
				just_test := True
				consume_option
			end

--		   TODO: Temporariily disabled, does not work yet (aleitner)
-- 			if match_long_option ("debug") then
-- 				is_debug_mode_enabled := True
-- 				consume_option
-- 			end
--
			if match_long_option ("define") then
				if not is_next_option_long_option or not has_next_option_value then
					error_handler.report_missing_command_line_parameter_value_error ("--define")
					error_handler.report_usage_error
					Exceptions.die (1)
				else
					defined_variables := next_option_value
				end
				consume_option
			end

			if match_long_option ("output-dir") then
				if not is_next_option_long_option or not has_next_option_value then
					error_handler.report_missing_command_line_parameter_value_error ("--output-dir")
					error_handler.report_usage_error
					Exceptions.die (1)
				else
					output_dirname := next_option_value
				end
				consume_option
			end

			if match_long_option ("time-out") then
				if not is_next_option_long_option or not has_next_option_value then
					error_handler.report_missing_command_line_parameter_value_error ("--time-out")
					error_handler.report_usage_error
					Exceptions.die (1)
				else
					if not next_option_value.is_integer then
						error_handler.report_invalid_time_out_value (next_option_value)
						Exceptions.die (1)
					else
						create time_out.make (0, next_option_value.to_integer, 0)
					end
				end
				consume_option
			end

			if next_option_position <= Arguments.argument_count then
				ace_filename := Arguments.argument (next_option_position)
				next_option_position := next_option_position + 1
			else
				error_handler.report_missing_ace_filename_error
				error_handler.report_usage_error
				Exceptions.die (1)
			end

			from
				create class_names.make
			until
				next_option_position > Arguments.argument_count
			loop
				class_names.force_last (Arguments.argument (next_option_position))
				next_option_position := next_option_position + 1
			end

		end

end
